'use strict';

class Appbar extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      collapseId: `collapse-${Math.floor(Math.random() * 0xFFFF)}`,
    }
  }

  getAppSwitcher() {
    return (this.props.appSwitcher.hide) ?
      <AppSwitcherDisabled {...this.props.appSwitcher} /> :
      <AppSwitcher {...this.props.appSwitcher} />;
  }

  render() {
    return (
      <nav className={`appbar navbar navbar-default navbar-white navbar-fixed-top`}>
        <div className="container-fluid">
          <div className="navbar-header">
            <button type="button" className="navbar-toggle collapsed" data-target={`#${this.state.collapseId}`} data-toggle="collapse">
              <span className="sr-only">Toggle navigation</span>
              <span className="icon-bar" />
              <span className="icon-bar" />
              <span className="icon-bar" />
            </button>
            {this.getAppSwitcher()}
          </div>
          <div className="collapse navbar-collapse" id={this.state.collapseId}>
            <style>{` body { padding-top: ${this.props.padding}px; } `}</style>
            <AppbarNotifications {...this.props.notifications} />
            <AppbarUserMenu {...this.props.userMenu} />
            <div className="appbar-content" dangerouslySetInnerHTML={{__html: this.props.content}} />
          </div>
        </div>
      </nav>
    );
  }
}

Appbar.propTypes = {
  brand: React.PropTypes.string,
  brandUrl: React.PropTypes.string,
  content: React.PropTypes.string,
  padding: React.PropTypes.number,
  appSwitcher: React.PropTypes.object,
  userMenu: React.PropTypes.object,
  notifications: React.PropTypes.object,
}

Appbar.defaultProps = {
  padding: 70,
  appSwitcher: {},
  userMenu: {},
  notifications: {},
}

// ====

class AppSwitcherDisabled extends React.Component {
  render() {
    return (
      <a className="navbar-brand" href={this.props.brandUrl}>
        <span className="appbar-sitename">{this.props.siteName}</span>
        <span className="appbar-brand" style={{color: this.props.brandColor}}>{this.props.brand}</span>
      </a>
    );
  }
}

// ====

class AppSwitcher extends React.Component {
  componentDidMount() {
    let $component = $(ReactDOM.findDOMNode(this));
    let width = $component.find('.appbar-sitename').width() - 8;
    $component.find('.appbar-menu-arrow').css({left: width + 'px'});
  }

  render() {
    return (
      <div className="navbar-brand app-switcher">
        <a href="#" className="dropdown-toggle" data-toggle="dropdown">
          <span className="appbar-sitename">{this.props.siteName}</span>
          <span className="appbar-brand" style={{color: this.props.brandColor}}>
            <span>{this.props.brand} </span>
            <i className="fa fa-angle-down" />
          </span>
        </a>

        <div className="dropdown-menu">
          <span className="appbar-menu-arrow" />
          <HomeScreen apps={this.props.apps} />
        </div>
      </div>
    );
  }
}

// ====

class HomeScreen extends React.Component {
  render() {
    return (
      <div className="row home-screen">
        {this.props.apps && this.props.apps.map && this.props.apps.map((props, key) => <HomeScreenApp {...props} key={key} />)}
      </div>
    );
  }
}

// ====

class HomeScreenApp extends React.Component {
  render() {
    return (
      <div className="home-screen-app col-xs-3 col-sm-2 text-center">
        <a href={this.props.url}>
          <img src={this.props.iconUrl} className={`img-responsive center-block ${this.props.iconRounded && 'icon-rounded'} ${this.props.iconBorder && 'icon-border'}`} />
          <span className="app-name">{this.props.name}</span>
        </a>
      </div>
    );
  }
}

HomeScreenApp.defaultProps = {
  iconBorder: false,
  iconRounded: true,
}

// ====

class AppbarUserMenu extends React.Component {
  render() {
    if (this.props.hide) return <span />;

    return (
      <ul className="appbar-user-menu nav navbar-nav navbar-right">
        <li className="dropdown dropdown-arrow">
          <a href="#" className="dropdown-toggle" data-toggle="dropdown">
            <span className="avatar-wrapper">
              <img src={this.props.avatarUrl} className="avatar" />
              <span className="visible-xs-inline">{this.props.userName}</span>
            </span>
          </a>

          <ul className="dropdown-menu" dangerouslySetInnerHTML={{__html: this.props.items}} />
        </li>
      </ul>
    );
  }
}

// ====

class AppbarNotifications extends React.Component {
  constructor(props) {
    super(props);
    this.state = {notifications: [], unseenCount: 0};
  }

  componentDidMount() {
    if ( ! this.props.hide && this.props.url) {
      this.fetchNotifications();
    }
  }

  componentWillUnmount() {
    if (this.timeoutFn) {
      clearTimeout(this.timeoutFn);
    }
  }

  fetchNotifications() {
    $.getJSON(this.props.url, notifications => {
      let unseenCount = notifications.filter(n => !n.seen).length;
      this.setState({notifications, unseenCount});
      this.timeoutFn = setTimeout(this.fetchNotifications.bind(this), 30000);
    });
  }

  handleDropdownToggle(e) {
    if (this.state.unseenCount > 0) {
      let unseenCount = 0;
      let ids = [];
      let notifications = this.state.notifications.map(n => {
        ids.push(n.id);
        n.seen = true;
        return n;
      });
      $.ajax({method: 'PUT', url: `${this.props.url}/mark_seen`, dataType: 'json', data: {ids: ids}});
      this.setState({notifications, unseenCount});
    }
  }

  render() {
    if (this.props.hide) return <span />;
    let unseenCount = this.state.unseenCount;
    let hasNewClass = unseenCount > 0 ? 'has-unseen' : '';

    return (
      <ul className="appbar-notifications nav navbar-nav navbar-right">
        <li className="dropdown dropdown-arrow">
          <a href="#" onClick={this.handleDropdownToggle.bind(this)} className={`dropdown-toggle ${hasNewClass}`} data-toggle="dropdown">
            <span className="icon-wrapper">
              <i className="fa fa-globe" />
              <span className="notifications-counter">{unseenCount}</span>
            </span>
          </a>

          <div className="dropdown-menu">
            <div className="notifications-wrapper">
              {this.state.notifications.map((n, i) => <AppbarNotification notiticatonsUrl={this.props.url} key={i} {...n} />)}
            </div>
          </div>
        </li>
      </ul>
    );
  }
}

// ====

class AppbarNotification extends React.Component {
  handleClick() {
    if ( ! this.props.clicked) {
      $.ajax({
        method: 'PUT',
        url: `${this.props.notiticatonsUrl}/${this.props.id}`,
        dataType: 'json',
      });
    }
  }

  render() {
    let unclickedClass = this.props.clicked ? '' : 'unclicked';
    let date = (typeof moment == 'function') && this.props.date ? moment(this.props.date).fromNow() : this.props.date;

    return (
      <a href={this.props.url} onClick={this.handleClick.bind(this)} className={`appbar-notification ${unclickedClass} clearfix`}>
        <span className="media-left">
          <img className="notification-image" src={this.props.imageUrl} />
        </span>
        <span className="media-body">
          <span>{this.props.message}</span>
          <div className="notification-date">{date}</div>
        </span>
      </a>
    );
  }
}
